@@ -991,7 +991,7 @@ static PyObject *WraptObjectProxy_enter(
 
     result = PyObject_Call(method, args, kwds);
 
-    Py_XDECREF(method);
+    Py_DECREF(method);
 
     return result;
 }
@@ -1016,7 +1016,7 @@ static PyObject *WraptObjectProxy_exit(
 
     result = PyObject_Call(method, args, kwds);
 
-    Py_XDECREF(method);
+    Py_DECREF(method);
 
     return result;
 }
@@ -1301,6 +1301,8 @@ static int WraptObjectProxy_setattro(
 
         return PyObject_GenericSetAttr((PyObject *)self, name, value);
     }
+    else if (!match)
+        PyErr_Clear();
 
     Py_XDECREF(match);
 
@@ -1312,15 +1314,11 @@ static int WraptObjectProxy_setattro(
 #endif
     }
 
-    if (PyObject_RichCompareBool(name, wrapped_str, Py_EQ) == 1) {
-        Py_DECREF(wrapped_str);
-
+    if (PyObject_RichCompareBool(name, wrapped_str, Py_EQ) == 1)
         return PyObject_GenericSetAttr((PyObject *)self, name, value);
-    }
 
-    if (PyObject_HasAttr((PyObject *)Py_TYPE(self), name)) {
+    if (PyObject_HasAttr((PyObject *)Py_TYPE(self), name))
         return PyObject_GenericSetAttr((PyObject *)self, name, value);
-    }
 
     if (!self->wrapped) {
       PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
@@ -1728,6 +1726,9 @@ static PyObject *WraptFunctionWrapperBase_descr_get(
         descriptor = (Py_TYPE(self->object_proxy.wrapped)->tp_descr_get)(
                 self->object_proxy.wrapped, obj, type);
 
+        if (!descriptor)
+            return NULL;
+
         if (Py_TYPE(self) != &WraptFunctionWrapper_Type) {
             bound_type = PyObject_GenericGetAttr((PyObject *)self,
                     bound_type_str);
@@ -1736,18 +1737,16 @@ static PyObject *WraptFunctionWrapperBase_descr_get(
                 PyErr_Clear();
         }
 
-        if (descriptor) {
-            if (obj == NULL)
-                obj = Py_None;
+        if (obj == NULL)
+            obj = Py_None;
 
-            result = PyObject_CallFunctionObjArgs(bound_type ? bound_type :
-                    (PyObject *)&WraptBoundFunctionWrapper_Type, descriptor,
-                    obj, self->wrapper, self->enabled, self->binding,
-                    self, NULL);
-        }
+        result = PyObject_CallFunctionObjArgs(bound_type ? bound_type :
+                (PyObject *)&WraptBoundFunctionWrapper_Type, descriptor,
+                obj, self->wrapper, self->enabled, self->binding,
+                self, NULL);
 
         Py_XDECREF(bound_type);
-        Py_XDECREF(descriptor);
+        Py_DECREF(descriptor);
 
         return result;
     }
@@ -1785,6 +1784,9 @@ static PyObject *WraptFunctionWrapperBase_descr_get(
 
         Py_DECREF(wrapped);
 
+        if (!descriptor)
+            return NULL;
+
         if (Py_TYPE(self->parent) != &WraptFunctionWrapper_Type) {
             bound_type = PyObject_GenericGetAttr((PyObject *)self->parent,
                     bound_type_str);
@@ -1793,18 +1795,16 @@ static PyObject *WraptFunctionWrapperBase_descr_get(
                 PyErr_Clear();
         }
 
-        if (descriptor) {
-            if (obj == NULL)
-                obj = Py_None;
+        if (obj == NULL)
+            obj = Py_None;
 
-            result = PyObject_CallFunctionObjArgs(bound_type ? bound_type :
-                    (PyObject *)&WraptBoundFunctionWrapper_Type, descriptor,
-                    obj, self->wrapper, self->enabled, self->binding,
-                    self->parent, NULL);
-        }
+        result = PyObject_CallFunctionObjArgs(bound_type ? bound_type :
+                (PyObject *)&WraptBoundFunctionWrapper_Type, descriptor,
+                obj, self->wrapper, self->enabled, self->binding,
+                self->parent, NULL);
 
         Py_XDECREF(bound_type);
-        Py_XDECREF(descriptor);
+        Py_DECREF(descriptor);
 
         return result;
     }
@@ -1997,11 +1997,6 @@ static PyObject *WraptBoundFunctionWrapper_call(
 #endif
     }
 
-    if (!kwds) {
-        param_kwds = PyDict_New();
-        kwds = param_kwds;
-    }
-
     /* 
     * We need to do things different depending on whether we are likely
     * wrapping an instance method vs a static method or class method.
@@ -2021,54 +2016,62 @@ static PyObject *WraptBoundFunctionWrapper_call(
              */
 
             PyObject *module = NULL;
+            PyObject *dict = NULL;
             PyObject *partial = NULL;
-            PyObject *object = NULL;
 
             module = PyImport_ImportModule("functools");
 
-            if (module) {
-                PyObject *dict = NULL;
+            if (!module)
+                return NULL;
 
-                dict = PyModule_GetDict(module);
-                partial = PyDict_GetItemString(dict, "partial");
+            dict = PyModule_GetDict(module);
+            partial = PyDict_GetItemString(dict, "partial");
 
+            if (!partial) {
                 Py_DECREF(module);
+                return NULL;
             }
 
-            if (!partial)
-                return NULL;
+            Py_INCREF(partial);
+            Py_DECREF(module);
 
             instance = PyTuple_GetItem(args, 0);
 
-            if (!instance)
+            if (!instance) {
+                Py_DECREF(partial);
                 return NULL;
+            }
 
-            object = PyObject_CallFunctionObjArgs(partial,
+            wrapped = PyObject_CallFunctionObjArgs(partial,
                     self->object_proxy.wrapped, instance, NULL);
 
-            if (!object)
-                return NULL;
+            Py_DECREF(partial);
 
-            wrapped = object;
+            if (!wrapped)
+                return NULL;
 
             param_args = PyTuple_GetSlice(args, 1, PyTuple_Size(args));
-            if (!param_args)
+
+            if (!param_args) {
+                Py_DECREF(wrapped);
                 return NULL;
+            }
+
             args = param_args;
         }
         else
             instance = self->instance;
 
-        if (!kwds) {
-            param_kwds = PyDict_New();
-            kwds = param_kwds;
-        }
-
         if (!wrapped) {
             Py_INCREF(self->object_proxy.wrapped);
             wrapped = self->object_proxy.wrapped;
         }
 
+        if (!kwds) {
+            param_kwds = PyDict_New();
+            kwds = param_kwds;
+        }
+
         result = PyObject_CallFunctionObjArgs(self->wrapper, wrapped,
                 instance, args, kwds, NULL);
 
@@ -2103,6 +2106,11 @@ static PyObject *WraptBoundFunctionWrapper_call(
             instance = Py_None;
         }
 
+        if (!kwds) {
+            param_kwds = PyDict_New();
+            kwds = param_kwds;
+        }
+
         result = PyObject_CallFunctionObjArgs(self->wrapper,
                 self->object_proxy.wrapped, instance, args, kwds, NULL);
 
